@@ -14,28 +14,51 @@ import threading
 from robotframework_debug_adapter.constants import (
     STATE_RUNNING,
     STATE_PAUSED,
-    REASON_BREAKPOINT,
-    STEP_IN,
-    REASON_STEP,
-    STEP_NEXT,
-    STEP_OUT,
+    ReasonEnum,
+    StepEnum,
 )
 import itertools
-from functools import partial
-from os.path import os
-from robocode_ls_core.robotframework_log import get_logger
+from functools import partial, lru_cache
+import os.path
+from robocode_ls_core.robotframework_log import get_logger, get_log_level
 from collections import namedtuple
-from robocode_ls_core.constants import IS_PY2
 import weakref
-import sys
+from robotframework_debug_adapter.protocols import (
+    IRobotDebugger,
+    INextId,
+    IRobotBreakpoint,
+    IBusyWait,
+    IEvaluationInfo,
+)
+from typing import Optional, List, Iterable, Union, Any, Dict, FrozenSet
+from robocode_ls_core.basic import implements
+from robotframework_debug_adapter.dap.dap_schema import (
+    StackFrame,
+    Scope,
+    Source,
+    Variable,
+)
+
+from robotframework_ls.impl.robot_constants import BUILTIN_VARIABLES
+
+
+@lru_cache(None)
+def get_builtin_normalized_names() -> FrozenSet[str]:
+    from robotframework_ls.impl.text_utilities import normalize_robot_name
+
+    normalized = list()
+    for k, _ in BUILTIN_VARIABLES:
+        normalized.append(normalize_robot_name(k))
+    return frozenset(normalized)
+
 
 log = get_logger(__name__)
 
-next_id = partial(next, itertools.count(1))
+next_id: INextId = partial(next, itertools.count(1))
 
 
 class RobotBreakpoint(object):
-    def __init__(self, lineno):
+    def __init__(self, lineno: int):
         """
         :param int lineno:
             1-based line for the breakpoint.
@@ -45,17 +68,27 @@ class RobotBreakpoint(object):
 
 class BusyWait(object):
     def __init__(self):
-        self._event = threading.Event()
         self.before_wait = []
+        self.waited = 0
+        self.proceeded = 0
+        self._condition = threading.Condition()
 
-    def wait(self):
+    @implements(IBusyWait.pre_wait)
+    def pre_wait(self):
         for c in self.before_wait:
             c()
-        self._event.wait()
 
+    @implements(IBusyWait.wait)
+    def wait(self):
+        self.waited += 1
+        with self._condition:
+            self._condition.wait()
+
+    @implements(IBusyWait.proceed)
     def proceed(self):
-        self._event.set()
-        self._event.clear()
+        self.proceeded += 1
+        with self._condition:
+            self._condition.notify_all()
 
 
 class _BaseObjectToDAP(object):
@@ -63,7 +96,7 @@ class _BaseObjectToDAP(object):
     Base class for classes which converts some object to the DAP.
     """
 
-    def compute_as_dap(self):
+    def compute_as_dap(self) -> List[Variable]:
         return []
 
 
@@ -75,8 +108,7 @@ class _ArgsAsDAP(_BaseObjectToDAP):
     def __init__(self, keyword_args):
         self._keyword_args = keyword_args
 
-    def compute_as_dap(self):
-        from robotframework_debug_adapter.dap.dap_schema import Variable
+    def compute_as_dap(self) -> List[Variable]:
         from robotframework_debug_adapter.safe_repr import SafeRepr
 
         lst = []
@@ -86,33 +118,57 @@ class _ArgsAsDAP(_BaseObjectToDAP):
         return lst
 
 
-class _VariablesAsDAP(_BaseObjectToDAP):
+class _NonBuiltinVariablesAsDAP(_BaseObjectToDAP):
     """
     Provides variables as DAP variables.
     """
 
     def __init__(self, variables):
         self._variables = variables
+        self._builtins = get_builtin_normalized_names()
 
-    def compute_as_dap(self):
-        from robotframework_debug_adapter.dap.dap_schema import Variable
+    def compute_as_dap(self) -> List[Variable]:
         from robotframework_debug_adapter.safe_repr import SafeRepr
 
         variables = self._variables
         as_dct = variables.as_dict()
         lst = []
         safe_repr = SafeRepr()
+
         for key, val in as_dct.items():
-            lst.append(Variable(safe_repr(key), safe_repr(val), variablesReference=0))
+            if self._accept(key):
+                lst.append(
+                    Variable(safe_repr(key), safe_repr(val), variablesReference=0)
+                )
         return lst
 
+    def _accept(self, k: str) -> bool:
+        from robotframework_ls.impl.text_utilities import normalize_robot_name
+
+        if normalize_robot_name(k) in self._builtins:
+            return False
+        else:
+            return True
+
+
+class _BuiltinsAsDAP(_NonBuiltinVariablesAsDAP):
+    """
+    Provides variables as DAP variables.
+    """
+
+    def _accept(self, k: str) -> bool:
+        return not _NonBuiltinVariablesAsDAP._accept(self, k)
+
 
 class _BaseFrameInfo(object):
     @property
     def dap_frame(self):
         raise NotImplementedError("Not implemented in: %s" % (self.__class__,))
 
-    def get_scopes(self):
+    def get_scopes(self) -> List[Scope]:
+        raise NotImplementedError("Not implemented in: %s" % (self.__class__,))
+
+    def get_type_name(self):
         raise NotImplementedError("Not implemented in: %s" % (self.__class__,))
 
 
@@ -125,9 +181,12 @@ class _SuiteFrameInfo(_BaseFrameInfo):
     def dap_frame(self):
         return self._dap_frame
 
-    def get_scopes(self):
+    def get_scopes(self) -> List[Scope]:
         return []
 
+    def get_type_name(self):
+        return "Suite"
+
 
 class _TestFrameInfo(_BaseFrameInfo):
     def __init__(self, stack_list, dap_frame):
@@ -138,9 +197,12 @@ class _TestFrameInfo(_BaseFrameInfo):
     def dap_frame(self):
         return self._dap_frame
 
-    def get_scopes(self):
+    def get_scopes(self) -> List[Scope]:
         return []
 
+    def get_type_name(self):
+        return "Test"
+
 
 class _KeywordFrameInfo(_BaseFrameInfo):
     def __init__(self, stack_list, dap_frame, keyword, variables):
@@ -150,21 +212,31 @@ class _KeywordFrameInfo(_BaseFrameInfo):
         self._scopes = None
         self._variables = variables
 
+    @property
+    def keyword(self):
+        return self._keyword
+
+    @property
+    def variables(self):
+        return self._variables
+
     @property
     def dap_frame(self):
         return self._dap_frame
 
-    def get_scopes(self):
+    def get_type_name(self):
+        return "Keyword"
+
+    def get_scopes(self) -> List[Scope]:
         if self._scopes is not None:
             return self._scopes
         stack_list = self._stack_list()
         if stack_list is None:
             return []
 
-        from robotframework_debug_adapter.dap.dap_schema import Scope
-
-        locals_variables_reference = next_id()
-        vars_variables_reference = next_id()
+        locals_variables_reference: int = next_id()
+        vars_variables_reference: int = next_id()
+        builtions_variables_reference: int = next_id()
         scopes = [
             Scope("Variables", vars_variables_reference, expensive=False),
             Scope(
@@ -173,6 +245,7 @@ class _KeywordFrameInfo(_BaseFrameInfo):
                 expensive=False,
                 presentationHint="locals",
             ),
+            Scope("Builtins", builtions_variables_reference, expensive=False),
         ]
 
         try:
@@ -186,7 +259,10 @@ class _KeywordFrameInfo(_BaseFrameInfo):
         # ctx.namespace.get_library_instances()
 
         stack_list.register_variables_reference(
-            vars_variables_reference, _VariablesAsDAP(self._variables)
+            vars_variables_reference, _NonBuiltinVariablesAsDAP(self._variables)
+        )
+        stack_list.register_variables_reference(
+            builtions_variables_reference, _BuiltinsAsDAP(self._variables)
         )
         self._scopes = scopes
         return self._scopes
@@ -199,32 +275,34 @@ class _StackInfo(object):
     """
 
     def __init__(self):
-        self._frame_id_to_frame_info = {}
+        self._frame_id_to_frame_info: Dict[int, _BaseFrameInfo] = {}
         self._dap_frames = []
         self._ref_id_to_children = {}
 
-    def iter_frame_ids(self):
-        return iter(self._frame_id_to_frame_info.keys())
+    def iter_frame_ids(self) -> Iterable[int]:
+        """
+        Access to list(int) where iter_frame_ids[0] is the current frame
+        where we're stopped (topmost frame).
+        """
+        return (x.id for x in self._dap_frames)
 
     def register_variables_reference(self, variables_reference, children):
         self._ref_id_to_children[variables_reference] = children
 
-    def add_keyword_entry_stack(self, keyword, filename, variables):
-        from robotframework_debug_adapter.dap import dap_schema
-
-        frame_id = next_id()
+    def add_keyword_entry_stack(self, keyword, filename: str, variables) -> int:
+        frame_id: int = next_id()
         try:
             name = str(keyword).strip()
             if not name:
                 name = keyword.__class__.__name__
         except:
             name = "<Unable to get keyword name>"
-        dap_frame = dap_schema.StackFrame(
+        dap_frame = StackFrame(
             frame_id,
             name=name,
             line=keyword.lineno or 1,
             column=0,
-            source=dap_schema.Source(name=os.path.basename(filename), path=filename),
+            source=Source(name=os.path.basename(filename), path=filename),
         )
         self._dap_frames.append(dap_frame)
         self._frame_id_to_frame_info[frame_id] = _KeywordFrameInfo(
@@ -232,25 +310,23 @@ class _StackInfo(object):
         )
         return frame_id
 
-    def add_suite_entry_stack(self, name, filename):
-        from robotframework_debug_adapter.dap import dap_schema
-
-        frame_id = next_id()
-        dap_frame = dap_schema.StackFrame(
+    def add_suite_entry_stack(self, name: str, filename: str) -> int:
+        frame_id: int = next_id()
+        dap_frame = StackFrame(
             frame_id,
             name=name,
             line=1,
             column=0,
-            source=dap_schema.Source(name=os.path.basename(filename), path=filename),
+            source=Source(name=os.path.basename(filename), path=filename),
         )
         self._dap_frames.append(dap_frame)
         self._frame_id_to_frame_info[frame_id] = _SuiteFrameInfo(self, dap_frame)
         return frame_id
 
-    def add_test_entry_stack(self, name, filename, lineno):
+    def add_test_entry_stack(self, name: str, filename: str, lineno: int) -> int:
         from robotframework_debug_adapter.dap import dap_schema
 
-        frame_id = next_id()
+        frame_id: int = next_id()
         dap_frame = dap_schema.StackFrame(
             frame_id,
             name=name,
@@ -263,7 +339,11 @@ class _StackInfo(object):
         return frame_id
 
     @property
-    def dap_frames(self):
+    def dap_frames(self) -> List[StackFrame]:
+        """
+        Access to list(StackFrame) where dap_frames[0] is the current frame
+        where we're stopped (topmost frame).
+        """
         return self._dap_frames
 
     def get_scopes(self, frame_id):
@@ -285,6 +365,129 @@ _SuiteEntry = namedtuple("_SuiteEntry", "name, source")
 _TestEntry = namedtuple("_TestEntry", "name, source, lineno")
 
 
+class InvalidFrameIdError(Exception):
+    pass
+
+
+class InvalidFrameTypeError(Exception):
+    pass
+
+
+class UnableToEvaluateError(Exception):
+    pass
+
+
+class EvaluationResult(Exception):
+    def __init__(self, result):
+        self.result = result
+
+
+class _EvaluationInfo(object):
+    def __init__(self, frame_id, expression):
+        from concurrent import futures
+
+        self.frame_id = frame_id
+        self.expression = expression
+        self.future = futures.Future()
+
+    def _do_eval(self, debugger_impl):
+        frame_id = self.frame_id
+        stack_info = debugger_impl._get_stack_info_from_frame_id(frame_id)
+
+        if stack_info is None:
+            raise InvalidFrameIdError(
+                "Unable to find frame id for evaluation: %s" % (frame_id,)
+            )
+
+        info = stack_info._frame_id_to_frame_info.get(frame_id)
+        if info is None:
+            raise InvalidFrameIdError(
+                "Unable to find frame info for evaluation: %s" % (frame_id,)
+            )
+
+        if not isinstance(info, _KeywordFrameInfo):
+            raise InvalidFrameTypeError(
+                "Can only evaluate at a Keyword context (current context: %s)"
+                % (info.get_type_name(),)
+            )
+        log.info("Doing evaluation in the Keyword context: %s", info.keyword)
+
+        from robotframework_ls.impl.text_utilities import is_variable_text
+
+        from robot.libraries.BuiltIn import BuiltIn  # type: ignore
+        from robot.api import get_model  # type: ignore
+        from robotframework_ls.impl import ast_utils
+
+        # We can't really use
+        # BuiltIn().evaluate(expression, modules, namespace)
+        # because we can't set the variable_store used with it
+        # (it always uses the latest).
+
+        variable_store = info.variables.store
+
+        if is_variable_text(self.expression):
+            try:
+                value = variable_store[self.expression[2:-1]]
+            except Exception:
+                pass
+            else:
+                return EvaluationResult(value)
+
+        # Do we want this?
+        # from robot.variables.evaluation import evaluate_expression
+        # try:
+        #     result = evaluate_expression(self.expression, variable_store)
+        # except Exception:
+        #     log.exception()
+        # else:
+        #     return EvaluationResult(result)
+
+        # Try to check if it's a KeywordCall.
+        s = """
+*** Test Cases ***
+Evaluation
+    %s
+""" % (
+            self.expression,
+        )
+        model = get_model(s)
+        usage_info = list(ast_utils.iter_keyword_usage_tokens(model))
+        if len(usage_info) == 1:
+            _stack, node, _token, name = next(iter(usage_info))
+
+            dap_frames = stack_info.dap_frames
+            if dap_frames:
+                top_frame_id = dap_frames[0].id
+                if top_frame_id != frame_id:
+                    if get_log_level() >= 2:
+                        log.debug(
+                            "Unable to evaluate.\nFrame id for evaluation: %r\nTop frame id: %r.\nDAP frames:\n%s",
+                            frame_id,
+                            top_frame_id,
+                            "\n".join(x.to_json() for x in dap_frames),
+                        )
+
+                    raise UnableToEvaluateError(
+                        "Keyword calls may only be evaluated at the topmost frame."
+                    )
+
+                return EvaluationResult(BuiltIn().run_keyword(name, *node.args))
+
+        raise UnableToEvaluateError("Unable to evaluate: %s" % (self.expression,))
+
+    def evaluate(self, debugger_impl):
+        """
+        :param _RobotDebuggerImpl debugger_impl:
+        """
+        try:
+            r = self._do_eval(debugger_impl)
+            self.future.set_result(r.result)
+        except Exception as e:
+            if get_log_level() >= 2:
+                log.exception("Error evaluating: %s", (self.expression,))
+            self.future.set_exception(e)
+
+
 class _RobotDebuggerImpl(object):
     """
     This class provides the main API to deal with debugging
@@ -292,35 +495,53 @@ class _RobotDebuggerImpl(object):
     """
 
     def __init__(self):
+        self.reset()
+
+    @implements(IRobotDebugger.reset)
+    def reset(self):
         from collections import deque
 
         self._filename_to_line_to_breakpoint = {}
         self.busy_wait = BusyWait()
 
         self._run_state = STATE_RUNNING
-        self._step_cmd = None
-        self._reason = None
+        self._step_cmd: StepEnum = StepEnum.STEP_NONE
+        self._reason: ReasonEnum = ReasonEnum.REASON_NOT_STOPPED
         self._next_id = next_id
         self._stack_ctx_entries_deque = deque()
         self._stop_on_stack_len = 0
 
-        self._tid_to_stack_list = {}
+        self._tid_to_stack_info: Dict[int, _StackInfo] = {}
         self._frame_id_to_tid = {}
+        self._evaluations = []
+        self._skip_breakpoints = 0
 
     @property
-    def stop_reason(self):
+    def stop_reason(self) -> ReasonEnum:
         return self._reason
 
-    def _get_stack_info(self, thread_id):
-        return self._tid_to_stack_list.get(thread_id)
+    def _get_stack_info_from_frame_id(self, frame_id) -> Optional[_StackInfo]:
+        thread_id = self._frame_id_to_tid.get(frame_id)
+        if thread_id is not None:
+            return self._get_stack_info(thread_id)
+        return None
 
-    def get_frames(self, thread_id):
+    def _get_stack_info(self, thread_id) -> Optional[_StackInfo]:
+        return self._tid_to_stack_info.get(thread_id)
+
+    def get_frames(self, thread_id) -> Optional[List[StackFrame]]:
         stack_info = self._get_stack_info(thread_id)
         if not stack_info:
             return None
         return stack_info.dap_frames
 
-    def get_scopes(self, frame_id):
+    def iter_frame_ids(self, thread_id) -> Iterable[int]:
+        stack_info = self._get_stack_info(thread_id)
+        if not stack_info:
+            return ()
+        return stack_info.iter_frame_ids()
+
+    def get_scopes(self, frame_id) -> Optional[List[Scope]]:
         tid = self._frame_id_to_tid.get(frame_id)
         if tid is None:
             return None
@@ -331,31 +552,26 @@ class _RobotDebuggerImpl(object):
         return stack_info.get_scopes(frame_id)
 
     def get_variables(self, variables_reference):
-        for stack_list in list(self._tid_to_stack_list.values()):
+        for stack_list in list(self._tid_to_stack_info.values()):
             variables = stack_list.get_variables(variables_reference)
             if variables is not None:
                 return variables
+        return None
 
-    def _get_filename(self, obj, msg):
+    def _get_filename(self, obj, msg) -> str:
         try:
             source = obj.source
             if source is None:
                 return u"None"
 
-            if IS_PY2 and isinstance(source, unicode):
-                source = source.encode(file_utils.file_system_encoding)
-
             filename, _changed = file_utils.norm_file_to_client(source)
-
-            if IS_PY2 and isinstance(filename, bytes):
-                source = source.decode(file_utils.file_system_encoding)
         except:
             filename = u"<Unable to get %s filename>" % (msg,)
             log.exception(filename)
 
         return filename
 
-    def _create_stack_info(self, thread_id):
+    def _create_stack_info(self, thread_id: int):
         stack_info = _StackInfo()
 
         for entry in reversed(self._stack_ctx_entries_deque):
@@ -388,14 +604,14 @@ class _RobotDebuggerImpl(object):
         for frame_id in stack_info.iter_frame_ids():
             self._frame_id_to_tid[frame_id] = thread_id
 
-        self._tid_to_stack_list[thread_id] = stack_info
+        self._tid_to_stack_info[thread_id] = stack_info
 
     def _dispose_stack_info(self, thread_id):
-        stack_list = self._tid_to_stack_list.pop(thread_id)
+        stack_list = self._tid_to_stack_info.pop(thread_id)
         for frame_id in stack_list.iter_frame_ids():
             self._frame_id_to_tid.pop(frame_id)
 
-    def wait_suspended(self, reason):
+    def wait_suspended(self, reason: ReasonEnum) -> None:
         from robotframework_debug_adapter.constants import MAIN_THREAD_ID
 
         log.info("wait_suspended", reason)
@@ -404,51 +620,88 @@ class _RobotDebuggerImpl(object):
             self._run_state = STATE_PAUSED
             self._reason = reason
 
+            self.busy_wait.pre_wait()
+
             while self._run_state == STATE_PAUSED:
                 self.busy_wait.wait()
 
-            if self._step_cmd == STEP_NEXT:
+                evaluations = self._evaluations
+                self._evaluations = []
+
+                for evaluation in evaluations:  #: :type evaluation: _EvaluationInfo
+                    self._skip_breakpoints += 1
+                    try:
+                        evaluation.evaluate(self)
+                    finally:
+                        self._skip_breakpoints -= 1
+
+            if self._step_cmd == StepEnum.STEP_NEXT:
                 self._stop_on_stack_len = len(self._stack_ctx_entries_deque)
 
-            elif self._step_cmd == STEP_OUT:
+            elif self._step_cmd == StepEnum.STEP_OUT:
                 self._stop_on_stack_len = len(self._stack_ctx_entries_deque) - 1
 
         finally:
+            self._reason = ReasonEnum.REASON_NOT_STOPPED
             self._dispose_stack_info(MAIN_THREAD_ID)
 
-    def step_continue(self):
-        self._step_cmd = None
-        self._run_state = STATE_RUNNING
-        self.busy_wait.proceed()
-
-    def step_in(self):
-        self._step_cmd = STEP_IN
-        self._run_state = STATE_RUNNING
-        self.busy_wait.proceed()
-
-    def step_next(self):
-        self._step_cmd = STEP_NEXT
-        self._run_state = STATE_RUNNING
-        self.busy_wait.proceed()
-
-    def step_out(self):
-        self._step_cmd = STEP_OUT
-        self._run_state = STATE_RUNNING
-        self.busy_wait.proceed()
-
-    def set_breakpoints(self, filename, breakpoints):
+    @implements(IRobotDebugger.evaluate)
+    def evaluate(self, frame_id, expression) -> IEvaluationInfo:
         """
-        :param str filename:
-        :param list(RobotBreakpoint) breakpoints:
+        Asks something to be evaluated.
+        
+        This is an asynchronous operation and returns an _EvaluationInfo (to get
+        the result, access _EvaluationInfo.future.result())
+        
+        :param frame_id:
+        :param expression:
+        :return _EvaluationInfo:
         """
-        if isinstance(breakpoints, RobotBreakpoint):
-            breakpoints = (breakpoints,)
-        if IS_PY2:
-            if isinstance(filename, unicode):
-                filename = filename.encode(file_utils.file_system_encoding)
-        filename = file_utils.get_abs_path_real_path_and_base_from_file(filename)[0]
+        evaluation_info = _EvaluationInfo(frame_id, expression)
+        self._evaluations.append(evaluation_info)
+        self.busy_wait.proceed()
+        return evaluation_info
+
+    @implements(IRobotDebugger.step_continue)
+    def step_continue(self) -> None:
+        self._step_cmd = StepEnum.STEP_NONE
+        self._run_state = STATE_RUNNING
+        self.busy_wait.proceed()
+
+    @implements(IRobotDebugger.step_in)
+    def step_in(self) -> None:
+        self._step_cmd = StepEnum.STEP_IN
+        self._run_state = STATE_RUNNING
+        self.busy_wait.proceed()
+
+    @implements(IRobotDebugger.step_next)
+    def step_next(self) -> None:
+        self._step_cmd = StepEnum.STEP_NEXT
+        self._run_state = STATE_RUNNING
+        self.busy_wait.proceed()
+
+    @implements(IRobotDebugger.step_out)
+    def step_out(self) -> None:
+        self._step_cmd = StepEnum.STEP_OUT
+        self._run_state = STATE_RUNNING
+        self.busy_wait.proceed()
+
+    @implements(IRobotDebugger.set_breakpoints)
+    def set_breakpoints(
+        self,
+        filename: str,
+        breakpoints: Union[IRobotBreakpoint, Iterable[IRobotBreakpoint]],
+    ) -> None:
+        iter_in: Any
+        if isinstance(breakpoints, (list, tuple, set)):
+            iter_in = breakpoints
+        else:
+            iter_in = [breakpoints]
+        filename = file_utils.get_abs_path_real_path_and_base_from_file(filename)[1]
         line_to_bp = {}
-        for bp in breakpoints:
+
+        for bp in iter_in:
+            log.info("Set breakpoint in %s: %s", filename, bp.lineno)
             line_to_bp[bp.lineno] = bp
         self._filename_to_line_to_breakpoint[filename] = line_to_bp
 
@@ -457,31 +710,33 @@ class _RobotDebuggerImpl(object):
     def before_run_step(self, step_runner, step, name=None):
         ctx = step_runner._context
         self._stack_ctx_entries_deque.append(_StepEntry(ctx.variables.current, step))
+        if self._skip_breakpoints:
+            return
 
         try:
             lineno = step.lineno
             source = step.source
-            if IS_PY2 and isinstance(source, unicode):
-                source = source.encode(file_utils.file_system_encoding)
         except AttributeError:
             return
 
-        log.debug("run_step %s, %s - step: %s\n", step, lineno, self._step_cmd)
-        source = file_utils.get_abs_path_real_path_and_base_from_file(source)[0]
+        source = file_utils.get_abs_path_real_path_and_base_from_file(source)[1]
+        log.debug(
+            "run_step %s, %s - step: %s - %s\n", step, lineno, self._step_cmd, source
+        )
         lines = self._filename_to_line_to_breakpoint.get(source)
 
-        stop_reason = None
+        stop_reason: Optional[ReasonEnum] = None
         step_cmd = self._step_cmd
         if lines and step.lineno in lines:
-            stop_reason = REASON_BREAKPOINT
+            stop_reason = ReasonEnum.REASON_BREAKPOINT
 
         elif step_cmd is not None:
-            if step_cmd == STEP_IN:
-                stop_reason = REASON_STEP
+            if step_cmd == StepEnum.STEP_IN:
+                stop_reason = ReasonEnum.REASON_STEP
 
-            elif step_cmd in (STEP_NEXT, STEP_OUT):
+            elif step_cmd in (StepEnum.STEP_NEXT, StepEnum.STEP_OUT):
                 if len(self._stack_ctx_entries_deque) <= self._stop_on_stack_len:
-                    stop_reason = REASON_STEP
+                    stop_reason = ReasonEnum.REASON_STEP
 
         if stop_reason is not None:
             self.wait_suspended(stop_reason)
@@ -522,12 +777,30 @@ def _patch(
     setattr(execution_context_cls, method_name, new_method)
 
 
-def patch_execution_context():
-    try:
-        impl = patch_execution_context.impl
-    except AttributeError:
+class _DebuggerHolder(object):
+    _dbg: Optional[IRobotDebugger] = None
+
+
+def set_global_robot_debugger(dbg: IRobotDebugger):
+    _DebuggerHolder._dbg = dbg
+
+
+def get_global_robot_debugger() -> Optional[IRobotDebugger]:
+    return _DebuggerHolder._dbg
+
+
+def install_robot_debugger() -> IRobotDebugger:
+    """
+    Installs the robot debugger and registers it where needed. If a debugger
+    is currently installed, resets it (in this case, any existing session,
+    stack trace, breakpoints, etc. are reset).
+    """
+
+    impl = get_global_robot_debugger()
+
+    if impl is None:
         # Note: only patches once, afterwards, returns the same instance.
-        from robot.running.steprunner import StepRunner
+        from robot.running.steprunner import StepRunner  # type: ignore
         from robotframework_debug_adapter.listeners import DebugListener
 
         impl = _RobotDebuggerImpl()
@@ -539,6 +812,8 @@ def patch_execution_context():
         DebugListener.on_end_test.register(impl.end_test)
 
         _patch(StepRunner, impl, "run_step", impl.before_run_step, impl.after_run_step)
-        patch_execution_context.impl = impl
+        set_global_robot_debugger(impl)
+    else:
+        impl.reset()
 
-    return patch_execution_context.impl
+    return impl
