@@ -23,13 +23,7 @@ import itertools
 import json
 import os.path
 import threading
-from robocode_ls_core.constants import IS_PY2
-from robocode_ls_core.basic import py2_filesystem_encode, py2_filesystem_decode
-
-try:
-    import Queue as queue
-except ImportError:
-    import queue
+import queue
 
 log = get_logger(__name__)
 
@@ -98,10 +92,11 @@ class _DebugAdapterRobotTargetComm(threading.Thread):
             read_from = socket.makefile("rb")
             write_to = socket.makefile("wb")
 
+            debug_adapter_comm = self._weak_debug_adapter_comm()
             writer = self._writer_thread = threading.Thread(
                 target=writer_thread_no_auto_seq,
                 args=(write_to, self._write_to_robot_queue, "write to robot process"),
-                name="Write to robot",
+                name="Write to robot (_DebugAdapterRobotTargetComm)",
             )
             writer.daemon = True
 
@@ -110,10 +105,10 @@ class _DebugAdapterRobotTargetComm(threading.Thread):
                 args=(
                     read_from,
                     self._from_robot,
-                    self._write_to_robot_queue,  # Used for errors
+                    debug_adapter_comm.write_to_client_queue,  # Used for errors
                     b"read from robot process",
                 ),
-                name="Read from robot",
+                name="Read from robot (_DebugAdapterRobotTargetComm)",
             )
             reader.daemon = True
 
@@ -405,8 +400,6 @@ class LaunchProcess(object):
                 + [target]
             )
 
-        if IS_PY2:
-            cmdline = [py2_filesystem_encode(c) for c in cmdline]
         self._cmdline = cmdline
 
     @property
@@ -542,8 +535,6 @@ class LaunchProcess(object):
 
             debug_adapter_comm = weak_debug_adapter_comm()
             cmdline = self._cmdline
-            if IS_PY2:
-                cmdline = [py2_filesystem_decode(c) for c in cmdline]
             if debug_adapter_comm is not None:
                 debug_adapter_comm.write_to_client_message(
                     RunInTerminalRequest(
