@@ -73,15 +73,16 @@ def main(args=None, after_bind=lambda server: None, language_server_class=None):
         # Automatically add it to the path if __main__ is being executed.
         sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
         import robotframework_ls  # @UnusedImport
+    robotframework_ls.import_robocode_ls_core()
 
     from robotframework_ls.options import Setup, Options
-    from robotframework_ls.robotframework_log import (
+    from robocode_ls_core.robotframework_log import (
         configure_logger,
         log_args_and_python,
+        get_logger,
     )
-    from robotframework_ls.robotframework_log import get_logger
 
-    from robotframework_ls.python_ls import (
+    from robocode_ls_core.python_ls import (
         start_io_lang_server,
         start_tcp_lang_server,
         binary_stdio,
@@ -104,7 +105,7 @@ def main(args=None, after_bind=lambda server: None, language_server_class=None):
 
     configure_logger("lsp", verbose, log_file)
     log = get_logger("robotframework_ls.__main__")
-    log_args_and_python(log, original_args)
+    log_args_and_python(log, original_args, robotframework_ls.__version__)
 
     if args.tcp:
         start_tcp_lang_server(
