@@ -2,8 +2,6 @@ import os
 import sys
 from robotframework_ls.constants import NULL
 from robocode_ls_core.robotframework_log import get_logger
-from robocode_ls_core.constants import IS_PY2
-from robocode_ls_core.basic import py2_filesystem_encode
 
 log = get_logger(__name__)
 
@@ -368,10 +366,7 @@ class LibspecManager(object):
         """
         from robocode_ls_core import watchdog_wrapper
 
-        try:
-            from concurrent import futures
-        except ImportError:
-            from robocode_ls_core.libs_py2.concurrent import futures
+        from concurrent import futures
 
         from multiprocessing import cpu_count
 
@@ -681,9 +676,6 @@ class LibspecManager(object):
                 ):  # Could fail.
                     call.append(libspec_filename)
 
-                    if IS_PY2:
-                        call = [py2_filesystem_encode(c) for c in call]
-
                     mtime = -1
                     try:
                         mtime = os.path.getmtime(libspec_filename)
@@ -783,8 +775,6 @@ class LibspecManager(object):
 
         :rtype: LibraryDoc
         """
-        if IS_PY2:
-            libname = py2_filesystem_encode(libname)
         libname_lower = libname.lower()
         if libname_lower.endswith((".py", ".class", ".java")):
             libname_lower = os.path.splitext(libname)[0]
