@@ -3,7 +3,7 @@ def complete(completion_context):
     :param CompletionContext completion_context:
     """
     import itertools
-    from robotframework_ls.lsp import (
+    from robocode_ls_core.lsp import (
         TextEdit,
         Range,
         Position,
