@@ -1,8 +1,7 @@
 import logging
 from robocode_ls_core.client_base import LanguageServerClientBase
-import sys
-from robocode_ls_core.constants import IS_PY2
-from robocode_ls_core.basic import py2_filesystem_decode
+from robocode_ls_core.basic import implements
+from robocode_ls_core.protocols import ILanguageServerClient
 
 log = logging.getLogger(__name__)
 
@@ -15,6 +14,7 @@ class _LanguageServerClient(LanguageServerClientBase):
 
         self.DEFAULT_TIMEOUT = TIMEOUT
 
+    @implements(ILanguageServerClient.settings)
     def settings(self, settings):
         self.request(
             {
@@ -25,13 +25,11 @@ class _LanguageServerClient(LanguageServerClientBase):
             }
         )
 
+    @implements(ILanguageServerClient.initialize)
     def initialize(self, root_path, msg_id=None, process_id=None):
         from robocode_ls_core.uris import from_fs_path
 
         root_uri = from_fs_path(root_path)
-        if IS_PY2:
-            root_path = py2_filesystem_decode(root_path)
-            root_uri = py2_filesystem_decode(root_uri)
 
         msg_id = msg_id if msg_id is not None else self.next_id()
         msg = self.request(
@@ -88,11 +86,8 @@ class _LanguageServerClient(LanguageServerClientBase):
         assert "capabilities" in msg["result"]
         return msg
 
+    @implements(ILanguageServerClient.open_doc)
     def open_doc(self, uri, version=1, text=""):
-        """
-        :param text:
-            If None, the contents will be loaded from the disk.
-        """
         self.write(
             {
                 "jsonrpc": "2.0",
@@ -108,6 +103,7 @@ class _LanguageServerClient(LanguageServerClientBase):
             }
         )
 
+    @implements(ILanguageServerClient.change_doc)
     def change_doc(self, uri, version, text):
         self.write(
             {
@@ -120,6 +116,7 @@ class _LanguageServerClient(LanguageServerClientBase):
             }
         )
 
+    @implements(ILanguageServerClient.get_completions)
     def get_completions(self, uri, line, col):
         return self.request(
             {
@@ -133,6 +130,7 @@ class _LanguageServerClient(LanguageServerClientBase):
             }
         )
 
+    @implements(ILanguageServerClient.request_source_format)
     def request_source_format(self, uri):
         return self.request(
             {
@@ -143,6 +141,7 @@ class _LanguageServerClient(LanguageServerClientBase):
             }
         )
 
+    @implements(ILanguageServerClient.find_definitions)
     def find_definitions(self, uri, line, col):
         return self.request(
             {
@@ -155,3 +154,14 @@ class _LanguageServerClient(LanguageServerClientBase):
                 },
             }
         )
+
+    @implements(ILanguageServerClient.execute_command)
+    def execute_command(self, command, arguments):
+        return self.request(
+            {
+                "jsonrpc": "2.0",
+                "id": self.next_id(),
+                "method": "workspace/executeCommand",
+                "params": {"command": command, "arguments": arguments},
+            }
+        )
