@@ -22,22 +22,14 @@ https://github.com/Microsoft/vscode-uri/blob/e59cab84f5df6265aed18ae5f43552d3eef
 
 import re
 
-from robocode_ls_core.constants import IS_PY2, IS_WIN
-from robocode_ls_core.basic import py2_filesystem_encode
+from robocode_ls_core.constants import IS_WIN
 
-if IS_PY2:
-    from urlparse import (
-        urlparse as parse_urlparse,  # noqa
-        urlunparse as parse_urlunparse,  # noqa
-    )
-    from urllib import quote as parse_quote, unquote as parse_unquote  # noqa
-else:
-    from urllib.parse import (
-        urlparse as parse_urlparse,  # noqa
-        urlunparse as parse_urlunparse,  # noqa
-        quote as parse_quote,  # noqa
-        unquote as parse_unquote,  # noqa
-    )
+from urllib.parse import (
+    urlparse as parse_urlparse,  # noqa
+    urlunparse as parse_urlunparse,  # noqa
+    quote as parse_quote,  # noqa
+    unquote as parse_unquote,  # noqa
+)
 
 RE_DRIVE_LETTER_PATH = re.compile(r"^\/[a-zA-Z]:")
 
@@ -76,8 +68,6 @@ def _normalize_win_path(path):
 def from_fs_path(path):
     """Returns a URI for the given filesystem path."""
     try:
-        if IS_PY2:
-            path = py2_filesystem_encode(path)
         scheme = "file"
         params, query, fragment = "", "", ""
         path, netloc = _normalize_win_path(path)
@@ -95,8 +85,6 @@ def to_fs_path(uri):
     Will *not* look at the scheme of this URI.
     """
     try:
-        if IS_PY2:
-            uri = py2_filesystem_encode(uri)
         # scheme://netloc/path;parameters?query#fragment
         scheme, netloc, path, _params, _query, _fragment = urlparse(uri)
 
