@@ -21,8 +21,9 @@ import os
 from robocode_ls_core import uris
 from robocode_ls_core.uris import uri_scheme, to_fs_path
 from robocode_ls_core.robotframework_log import get_logger
-from robocode_ls_core.constants import IS_PY2
-from robocode_ls_core.basic import py2_filesystem_decode
+from robocode_ls_core.basic import implements
+from robocode_ls_core.protocols import IWorkspace, IDocument
+from typing import Optional
 
 log = get_logger(__name__)
 
@@ -48,8 +49,6 @@ class Workspace(object):
 
         if root_uri and root_uri not in self.folders:
             as_fs_path = uris.to_fs_path(root_uri)
-            if IS_PY2:
-                as_fs_path = py2_filesystem_decode(as_fs_path)
             name = os.path.basename(as_fs_path)
             self.add_folder(WorkspaceFolder(root_uri, name))
 
@@ -72,12 +71,8 @@ class Workspace(object):
     def folders(self):
         return self._folders
 
-    def get_document(self, doc_uri, accept_from_file):
-        """
-        Return a managed document if-present, otherwise, create one pointing at
-        the disk if accept_from_file == True (if the file exists, and we're able to
-        load it, otherwise, return None).
-        """
+    @implements(IWorkspace.get_document)
+    def get_document(self, doc_uri: str, accept_from_file: bool) -> Optional[IDocument]:
         doc = self._docs.get(doc_uri)
         if doc is None:
             if accept_from_file:
@@ -256,7 +251,8 @@ class Document(object):
     def source(self, source):
         self._source = source
 
-    def get_line(self, line):
+    @implements(IDocument.get_line)
+    def get_line(self, line: int) -> str:
         try:
             return self._lines[line].rstrip("\r\n")
         except IndexError:
